/*
 Copyright (c) 2021 Tarek Mohamed Abdalla <tarekkma@gmail.com>

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.ichi2.anki

import com.fasterxml.jackson.core.JsonFactory
import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper

/**
 * provide a singleton instances of serialization classes ({@link ObjectMapper}, {@link JsonFactory})
 */
object AnkiSerialization {
    /**
     * @return singleton of {@link ObjectMapper} used to bind json to java classes
     */
    @JvmStatic
    val objectMapper: ObjectMapper by lazy {
        ObjectMapper().configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
    }

    /**
     * @return singleton of {@link JsonFactory} used for json stream processing
     */
    @JvmStatic
    val factory: JsonFactory
        get() = objectMapper.factory
}
